package com.chelvc.framework.base.config;

import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.function.Predicate;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.web.servlet.WebMvcRegistrations;
import org.springframework.boot.env.EnvironmentPostProcessor;
import org.springframework.context.annotation.Import;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.core.env.PropertiesPropertySource;
import org.springframework.core.env.PropertySource;
import org.springframework.core.io.Resource;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

/**
 * 多服务部署配置
 *
 * @author Woody
 * @date 2024/4/3
 */
public class MultipleServerConfigurer implements WebMvcRegistrations, EnvironmentPostProcessor {
    /**
     * 判断是否启用多服务配置
     *
     * @param application Spring应用实例
     * @return true/false
     */
    private boolean isEnabled(@NonNull SpringApplication application) {
        Class<?> main = application.getMainApplicationClass();
        if (main != null) {
            Class<?>[] imports = ObjectUtils.ifNull(main.getAnnotation(Import.class), Import::value);
            if (ObjectUtils.notEmpty(imports)) {
                for (Class<?> clazz : imports) {
                    if (clazz == MultipleServerConfigurer.class) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public RequestMappingHandlerMapping getRequestMappingHandlerMapping() {
        List<Resource> resources = SpringUtils.getPropertyResources();
        Map<String, Predicate<Class<?>>> prefixes = Maps.newHashMapWithExpectedSize(resources.size());
        for (Resource resource : resources) {
            String server = ApplicationContextHolder.getApplicationName(resource);
            if (StringUtils.notEmpty(server)) {
                prefixes.put(server, clazz -> SpringUtils.isResourceClass(resource, clazz));
            }
        }
        RequestMappingHandlerMapping mapping = new RequestMappingHandlerMapping();
        mapping.setPathPrefixes(prefixes);
        return mapping;
    }

    @Override
    public void postProcessEnvironment(ConfigurableEnvironment environment, SpringApplication application) {
        if (!this.isEnabled(application)) {
            return;
        }

        Properties properties = null;
        Class<?> main = application.getMainApplicationClass();
        List<Resource> resources = SpringUtils.getPropertyResources(environment);
        for (Resource resource : resources) {
            // 排除当前应用配置
            if (main != null && SpringUtils.isResourceClass(resource, main)) {
                continue;
            }

            // 合并其他服务配置
            if (properties == null) {
                properties = SpringUtils.getResourceProperties(resource);
            } else {
                properties.putAll(SpringUtils.getResourceProperties(resource));
            }
        }
        if (ObjectUtils.notEmpty(properties)) {
            PropertySource<?> source = new PropertiesPropertySource("multipleServerProperties", properties);
            environment.getPropertySources().addLast(source);
        }
    }
}